function [A, Phi, Xi] = RCMF( Y, K, k, iterations )
%% This code is provided to demonstrate the working of RCMF algorithm appearing in the following paper:
% Naveed Akhtar and Ajmal Mian,'RCMF: Robust Constrained Matrix
% Factorization for Hyperspectral Unmixing', IEEE Transactions on
% Geosciences and Remote Sensing, 2017.

%% Description:
% Inputs:
%   - Y in R^(m x n): input image with each pixel/spectra a column
%   - K a scalar: Dictionary size required by the algo
%   - k a scalar: Sparsity level for the NSP algorithm.
%   - iterations: Number of iterations for the algorithm.
%__________________________________________________________________________


[m,n] = size(Y);
Yn = normc(Y);
Y = Yn;
%% Initialize
[Xi, Phi] = initilaizeMatrices(Y, K, n);
E = 1e-9*ones(1,n);
%% Main loop
    for i = 1:iterations
        disp(['iteration number: ' int2str(i)])
        [A,res_p,res_d] = CSUNSALONE(Phi,Y,'lambda',1e-3,'ADDONE','yes','POSITIVITY','yes','AL_iters',200,'TOL', eps,'verbose','no');
        Gamma = Y - (Phi*A);      
        delta_sqrt_inv = 1./(sqrt(max([E;sqrt(sum(Gamma.*Gamma))], [],1)));
        for ii = 1:K
            rho = (A(ii,:).*delta_sqrt_inv)';
            d = (1/(A(ii,:)*rho)) * (Gamma*rho) + Phi(:,ii);
            xi = SP_plus(k ,Y,Yn, d);  
            Xi(:,ii) = xi.x_hat;
            Gamma = Gamma + (Phi(:,ii) - Y * Xi(:,ii))* A(ii,:);         
        end    
        Phi = Y * Xi;
    end
end

function [Xi, Phi] = initilaizeMatrices(Y, p, n)
Xi = zeros(n,p);
for i = 1:p
    indi = randperm(n,1);
    Xi(indi,i) = 1;
end
Phi = Y * Xi;
end
function [z,res_p,res_d] = CSUNSALONE(M,y,varargin)
% Dislaimer: Most parts of this function are directly copied from the
% original code provided by the authors of SUnSAL.

[LM,p] = size(M);
[L,N] = size(y);
x0 = 0;

%--------------------------------------------------------------
% Read the optional parameters
%--------------------------------------------------------------
if (rem(length(varargin),2)==1)
    error('Optional parameters should always go by pairs');
else
    for i=1:2:(length(varargin)-1)
        switch upper(varargin{i})
            case 'AL_ITERS'
                AL_iters = round(varargin{i+1});
                if (AL_iters <= 0 )
                       error('AL_iters must a positive integer');
                end
            case 'LAMBDA'
                lambda = varargin{i+1};
                if (sum(sum(lambda < 0)) >  0 )
                       error('lambda must be positive');
                end
            case 'POSITIVITY'
                positivity = varargin{i+1};
            case 'ADDONE'
                addone = varargin{i+1};
            case 'TOL'
                tol = varargin{i+1};
            case 'VERBOSE'
                verbose = varargin{i+1};
            case 'X0'
                x0 = varargin{i+1};
                if (size(x0,1) ~= p) | (size(x0,1) ~= N)
                    error('initial X is  inconsistent with M or Y');
                end
            otherwise
                % Hmmm, something wrong with the parameter string
                error(['Unrecognized option: ''' varargin{i} '''']);
        end;
    end;
end

%---------------------------------------------
%  If lambda is scalar convert it into vector
%---------------------------------------------
Nlambda = size(lambda);
if Nlambda == 1
    % same lambda for all pixels
    lambda = lambda*ones(p,N);
elseif Nlambda ~= N
        error('Lambda size is inconsistent with the size of the data set');
else
  %each pixel has its own lambda
   lambda = repmat(lambda(:)',p,1);
end

% compute mean norm
  norm_y = sqrt(mean(mean(y.^2)));
% rescale M and Y and lambda
M = M/norm_y;
y = y/norm_y;
lambda = lambda/norm_y^2;

 
B = ones(1,p);
a = ones(1,N);



%%
%---------------------------------------------
%  Constants and initializations
%---------------------------------------------
mu_AL = 0.01;
mu = 10*mean(lambda(:)) + mu_AL;

%F = M'*M+mu*eye(p);
[UF,SF] = svd(M'*M);
sF = diag(SF);
IF = UF*diag(1./(sF+mu))*UF';
%IF = inv(F);
Aux = (IF*B')/(B*IF*B');
x_aux = Aux*a;
IF1 = (IF-Aux*B*IF);


yy = M'*y;

%%
%---------------------------------------------
%  Initializations
%---------------------------------------------

% no intial solution supplied
if x0 == 0
    x= IF*M'*y;
end

z = x;
% scaled Lagrange Multipliers
d  = 0*z;


%%
%---------------------------------------------
%  AL iterations - main body
%---------------------------------------------
tol1 = sqrt(N*p)*tol;
tol2 = sqrt(N*p)*tol;
i=1;
res_p = inf;
res_d = inf;
maskz = ones(size(z));
mu_changed = 0;
while (i <= AL_iters) && ((abs (res_p) > tol1) || (abs (res_d) > tol2)) 
    % save z to be used later
    if mod(i,10) == 1
        z0 = z;
    end
    % minimize with respect to z
    z =  soft(x-d,lambda/mu);
    % teste for positivity
    if strcmp(positivity,'yes')
       maskz = (z >= 0);
       z = z.*maskz; 
    end
    % teste for sum-to-one 
    if strcmp(addone,'yes')
       x = IF1*(yy+mu*(z+d))+x_aux;
    else
       x = IF*(yy+mu*(z+d));
    end
    
    % Lagrange multipliers update
    d = d -(x-z);

    % update mu so to keep primal and dual residuals whithin a factor of 10
    if mod(i,10) == 1
        % primal residue
        res_p = norm(x-z,'fro');
        % dual residue
        res_d = mu*norm(z-z0,'fro');
        if  strcmp(verbose,'yes')
            fprintf(' i = %f, res_p = %f, res_d = %f\n',i,res_p,res_d)
        end
        % update mu
        if res_p > 10*res_d
            mu = mu*2;
            d = d/2;
            mu_changed = 1;
        elseif res_d > 10*res_p
            mu = mu/2;
            d = d*2;
            mu_changed = 1;
        end
        if  mu_changed
           % update IF and IF1

           IF = UF*diag(1./(sF+mu))*UF';
           Aux = IF*B'/(B*IF*B');
           x_aux = Aux*a;
           IF1 = (IF-Aux*B*IF);
            mu_changed = 0;
            %mu
        end
        
        
    end
    
    i=i+1;
        
   
       
end

    
 
end
function Rec = SP_plus(p,D,Dn, y)
[m,N]=size(D);
in = 1;
cv = normc(y)'*Dn;
[cv_sort, cv_index] = sort(cv,'descend');
cv_index = sort( cv_index(1:p) );
Phi_x = D(:,cv_index);
Index_save(in,:) = cv_index;

x_p = lsqnonneg(Phi_x, y);
y_r = y - Phi_x*x_p;
norm_save(in) = norm(y_r);

while 1
   x_p_old = x_p;
   in = in+1;
   cv = normc(y_r)'*Dn;
   [cv_sort, cv_index] = sort(cv,'descend');
   cv_index = sort( cv_index(1:p) );
   cv_add = union(Index_save(in-1,:), cv_index);
   Phi_x = D(:,cv_add); 
   x_p = lsqnonneg(Phi_x, y);
   [x_p_sort, i_sort] = sort(x_p , 'descend' );
   cv_index = cv_add( i_sort(1:p) );
   cv_index = sort( cv_index );
   Phi_x = D(:,cv_index);
   Index_save(in,:)=cv_index;

   % calculate the residue
   x_p = lsqnonneg(Phi_x, y);
   
   y_r = y - Phi_x*x_p;
   norm_save(in) = norm(y_r);

   if ( norm_save(in)== 0 ) || ...
           (norm_save(in)/norm_save(in-1) >= 1)
       inda = Index_save(in,:);
       break;
   end
end
if (norm_save(in)/norm_save(in-1) > 1) 
    x_p = x_p_old; %
    inda = Index_save(in-1,:);
end
x_hat = zeros(N,1);
x_hat( inda ) = reshape(x_p,p,1);
Rec.T = Index_save;
Rec.x_hat = x_hat;
Rec.PResidue = norm_save;
end




