%% Demonstration of working of Robust Constrained Matrix Factorization (RCMF) algorithm 
% Please cite the following paper if you use any part of the code:
%
% Naveed Akhtar and Ajmal Mian,'RCMF: Robust Constrained Matrix
% Factorization for Hyperspectral Unmixing', IEEE Transactions on
% Geosciences and Remote Sensing, 2017.
%
%%----------To run-------------------------------------------------
% Add the folder to Matlab path and run this file
%
%%----------Description--------------------------------------------
% The implementation provides approximate solution to the following problem:
% min  ( || (Y - Y Xi A) Del^(-1/2) ||_F^2 + ||Del||_1 ) s.t. forall i,j
% ||a_j||_1 = 1 & xi_(j,i), a_(i,j) >= 0 & ||xi_i||_0 <= k, delta_j >=
% epsilon.

% In the above:
%       - Y in R^(m x n): input image pixels
%       - Xi in R^(n x K): The matrix recording combinations of pixels
%       forming the endmembers
%       - A in R^(K x n): Sparse codes
%       - Del in R^(n x n): Diagonal matrix with strictly positive entries.
% Please refer to the paper for the details 

% Author: Naveed Akhtar, 2017
%__________________________________________________________________________

%%-----------------Parameter Settings ------------------------
pixels = 1000;                      % Number of pixels in the synthetic image
outlierPercent = 3;                 % Percentage of pixels being converted to outliers
imgMaterials = 24;                  % Total materials allowed to be present (Should not exceed 24 for this code)
endMemberRange = [2 5];             % Range of endmembers to be mixed per pixel for synthetic image. Number chosen at random in this range for each pixel. 
SNR = 50;                           % SNR for each pixel of synthetic image
library = 'forAVIRIS';              % Give path to the provided library if it is copied to another folder.
Noise = 'colored';                  % Noise type 'white' or 'colored'

%Algorithm parameters
K = 25;                             % Dictionary size 
iterations = 300;                   % Algorithm iterations
k = 7;                              % Sparsity level for the NSP algorithm
%__________________________________________________________________________


%%-----------------Synthetic image generation-----------------

[Y,A_GT,D] = createSyntheticImage( imgMaterials, pixels, endMemberRange, library ); % A_GT is the ground truth for abundaces
if strcmp (Noise, 'white')
    [Y] = whiteNoiseImage(Y,SNR);
end
if strcmp (Noise, 'colored')
    [Y] = coloredNoiseImage(Y, SNR);
end
[Y, outlier_ind] = outlierAddition(outlierPercent,Y);
%_______________________________________________________________


%%-----------------Run the algorithm---------------------------
tic
[A, Phi, Xi] = RCMF( Y, K, k, iterations );
time_RCMF = toc;
%_____________________________________________________________


%%-----------------Evaluate---------------------------------
spectra = normc(D);
[SAMs_RCMF, mathc_ind] = SAM_Analysis(spectra, Phi);
[RMSE_RCMF] = RMSE_Analysis(A_GT, A, mathc_ind, outlier_ind);
MeanSAM_RCMF = mean(SAMs_RCMF);

%%----------------Output--------------------------------------
disp(['Mean Spectral Angle Measurement for spectra: ' num2str(MeanSAM_RCMF)]);
disp(['Root Mean Square Error for abundances: ' num2str(RMSE_RCMF)]);
disp(['Total time taken by RCMF: ' num2str(time_RCMF/60) ' minutes']);
n = 5; figure, plot(1:size(spectra,1), 1/max(Phi(:,mathc_ind(n))) * Phi(:,mathc_ind(n)) , 'b', 1:size(spectra,1), 1/max(spectra(:,n))*spectra(:,n), 'r'), xlabel('Sample number'), ylabel('Normalized reflectance'),legend('Predicted', 'Ground truth'), title('Comparison for a random endmember'); 